@@ -926,7 +926,8 @@ void CmdExec::Reconfig(const char *name)
    verify_path_cached=ResMgr::QueryBool("cmd:verify-path-cached",c);
    verify_host=ResMgr::QueryBool("cmd:verify-host",c);
    verbose=ResMgr::QueryBool("cmd:verbose",0);
-   max_waiting=ResMgr::Query(queue_feeder?"cmd:queue-parallel":"cmd:parallel",c);
+   if(top_level || queue_feeder)
+      max_waiting=ResMgr::Query(queue_feeder?"cmd:queue-parallel":"cmd:parallel",c);
    if(name && !strcmp(name,"cmd:interactive"))
       SetInteractive();
    show_status=ResMgr::QueryBool("cmd:show-status",0);
@@ -1209,7 +1210,7 @@ void CmdExec::RegisterCommand(const char *name,cmd_creator_t creator,const char
       c->creator=creator;
       if(short_desc)
 	 c->short_desc=short_desc;
-      if(long_desc)
+      if(long_desc || strlen(c->long_desc)<2)
 	 c->long_desc=long_desc;
       return;
    }
@@ -1236,23 +1237,33 @@ const char *CmdExec::CmdByIndex(int i)
    return static_cmd_table[i].name;
 }
 
-Job *CmdExec::default_cmd()
+bool CmdExec::load_cmd_module(const char *op)
 {
-   const char *op=args->a0();
-#ifdef WITH_MODULES
    const char *modname=xstring::cat("cmd-",op,NULL);
+   if(module_init_preloaded(modname))
+      return true;
+#ifdef WITH_MODULES
    if(module_load(modname,0,0)==0)
    {
       eprintf("%s\n",module_error_message());
-      return 0;
+      return false;
    }
-   builtin=BUILTIN_EXEC_RESTART;
-   return this;
+   return true;
 #else
    eprintf(_("%s: command `%s' is not compiled in.\n"),op,op);
-   return 0;
+   return false;
 #endif
 }
+
+Job *CmdExec::default_cmd()
+{
+   const char *op=args->a0();
+   if(load_cmd_module(op)) {
+      builtin=BUILTIN_EXEC_RESTART;
+      return this;
+   }
+   return 0;
+}
 Job *CmdExec::builtin_local()
 {
    if(args->count()<2) {
