@@ -69,6 +69,7 @@ public:
    int	 prev_exit_code;
 
 private:
+   CmdExec *parent_exec;
    Buffer cmd_buf;
    bool partial_cmd;
    int alias_field; // length of expanded alias (and ttl for used_aliases)
@@ -176,7 +177,7 @@ public:
    void SuspendJob(Job *j);
 
    CmdExec(FileAccess *s,LocalDirectory *c);
-   CmdExec(const CmdExec *parent);
+   CmdExec(CmdExec *parent);
    ~CmdExec();
 
    bool Idle();	// when we have no command running and command buffer is empty
@@ -248,11 +249,12 @@ public:
    Job *builtin_queue_edit();
    Job *builtin_local();
 
+   bool load_cmd_module(const char *op);
    Job *default_cmd();
 
    void ChangeSession(FileAccess *new_session);
 
-   void print_cmd_help(const char *cmd);
+   bool print_cmd_help(const char *cmd);
    void print_cmd_index();
 
    static const char *CmdByIndex(int i);
