@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1999-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: CopyJob.h,v 1.29 2009/08/11 10:21:16 lav Exp $ */
-
 #ifndef COPYJOB_H
 #define COPYJOB_H
 
@@ -47,7 +44,7 @@ public:
    CopyJob(FileCopy *c1,const char *n,const char *op1);
    ~CopyJob();
 
-   void NoStatus() { no_status=true; }
+   void NoStatus(bool t=true) { no_status=t; }
    void NoStatusOnWrite() { no_status_on_write=true; }
    void ClearStatusOnWrite() { clear_status_on_write=true; }
    bool HasStatus() const { return !no_status; }
@@ -67,7 +64,8 @@ public:
    bool Error() { return c->Error(); }
    const char *ErrorText() { return c->ErrorText(); }
    double GetTimeSpent() { return c->GetTimeSpent(); }
-   virtual off_t GetBytesCount() { return c->GetBytesCount(); }
+   off_t GetBytesCount() { return c->GetBytesCount(); }
+   double GetTransferRate() { return c->GetTransferRate(); }
    off_t GetSize() { return c->GetSize(); }
    off_t GetPos()  { return c->GetPos(); }
    float GetRate() { return c->GetRate(); }
@@ -84,7 +82,7 @@ public:
 
    const char *Status(const StatusLine *s,bool base=false);
    void ShowRunStatus(const SMTaskRef<StatusLine>&);
-   void	PrintStatus(int,const char *);
+   xstring& FormatStatus(xstring&,int,const char *);
 
    const char *GetName() const { return name; }
    const char *GetDispName() const { return dispname; }
@@ -111,6 +109,7 @@ protected:
    int errors;
    int count;
    off_t bytes;
+   TimeDate transfer_start_ts;
    double time_spent;
    const char *op;
    bool no_status;
@@ -138,14 +137,14 @@ public:
 
    void SetCopyJobCreator(CopyJobCreator *c) { cj_new=c; }
 
-   void SayFinalWithPrefix(const char *p);
-   void SayFinal() { SayFinalWithPrefix(""); }
-   void	PrintStatus(int,const char *);
+   void SayFinal();
+   xstring& FormatFinalWithPrefix(xstring&,const char *);
+   xstring& FormatStatus(xstring&,int,const char *);
 
    void Ascii() { ascii=true; }
 
-   double GetTimeSpent() { return time_spent; }
-   off_t GetBytesCount() { return bytes; }
+   double GetTimeSpent() { return time_spent+(waiting.count()>0?now-transfer_start_ts:0.); }
+   off_t GetBytesCount() { return bytes+Job::GetBytesCount(); }
 };
 
 #endif // COPYJOB_H
