@@ -59,6 +59,7 @@ void FileAccess::Init()
    pos=0;
    mode=CLOSED;
    retries=0;
+   max_retries=0;
    opt_date=0;
    opt_size=0;
    fileset_for_info=0;
@@ -192,7 +193,6 @@ void FileAccess::Close()
    opt_size=0;
    fileset_for_info=0;
    retries=0;
-   max_retries=0;
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
    ascii=false;
@@ -684,13 +684,26 @@ FileAccess *SessionPool::Walk(int *n,const char *proto)
 
 void SessionPool::ClearAll()
 {
-   for(int n=0; n<pool_size; n++)
-   {
-      if(pool[n])
-      {
-	 SMTask::Delete(pool[n]);
-	 pool[n]=0;
+   int pass=0;
+   for(;;) {
+      int left=0;
+      for(int n=0; n<pool_size; n++) {
+	 if(!pool[n])
+	    continue;
+	 if(pass==0)
+	    pool[n]->Disconnect();
+	 if(!pool[n]->IsConnected()) {
+	    SMTask::Delete(pool[n]);
+	    pool[n]=0;
+	 } else {
+	    left++;
+	 }
       }
+      if(left==0)
+	 break;
+      SMTask::Schedule();
+      SMTask::Block();
+      pass++;
    }
 }
 
