@@ -30,6 +30,9 @@
 #include "FileSet.h"
 #include "ArgV.h"
 #include "ProtoLog.h"
+#include "xlist.h"
+#include "xmap.h"
+#include "Timer.h"
 
 #define FILE_END     ((off_t)-1L)
 #define UNKNOWN_POS  ((off_t)-1L)
@@ -39,6 +42,7 @@ class Glob;
 class NoGlob;
 class DirList;
 class FileAccessRef;
+class Buffer;
 
 class FileAccess : public SMTask, public ResClient, protected ProtoLog
 {
@@ -86,6 +90,7 @@ public:
       void Set(const char *new_path,bool new_is_file=false,const char *new_url=0,int device_prefix_len=0);
       void SetURL(const char *u) { url.set(u); }
       void Change(const char *new_path,bool new_is_file=false,const char *new_path_enc=0,int device_prefix_len=0);
+      const xstring& GetDirectory() const { return is_file?dirname(path):path; }
       void ExpandTilde(const Path &home);
       static void Optimize(xstring& p,int dev_prefix=0);
       void Optimize() { Optimize(path,device_prefix_len); }
@@ -110,7 +115,7 @@ protected:
    const char *default_cwd;
    Path	 home;
    Path	 cwd;
-   Path	 *new_cwd;
+   Ref<Path> new_cwd;
    xstring file;
    xstring file_url;
    xstring file1;
@@ -124,8 +129,9 @@ protected:
 
    FileSet *fileset_for_info;
 
-   time_t try_time;
+   Timer reconnect_timer;
    int retries;
+   int max_retries;
 
    bool	 mkdir_p;
 
@@ -169,8 +175,11 @@ protected:
    xstring_c entity_content_type;
    xstring_c entity_charset;
 
-   FileAccess *next;
-   static FileAccess *chain;
+   xstring_c last_disconnect_cause;
+
+   xlist<FileAccess> all_fa_node;
+   static xlist_head<FileAccess> all_fa;
+
    FileAccess *FirstSameSite() const { return NextSameSite(0); }
    FileAccess *NextSameSite(FileAccess *) const;
 
@@ -178,6 +187,8 @@ protected:
 
    int device_prefix_len(const char *path) const;
 
+   virtual ~FileAccess();
+
 public:
    virtual const char *GetProto() const = 0; // http, ftp, file etc
    bool SameProtoAs(const FileAccess *fa) const { return !strcmp(GetProto(),fa->GetProto()); }
@@ -194,6 +205,7 @@ public:
    const char  *GetConnectURL(int flags=0) const;
    const char  *GetFileURL(const char *file,int flags=0) const;
    enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
+   const char *GetLastDisconnectCause() const { return last_disconnect_cause; }
 
    void Connect(const char *h,const char *p);
    void ConnectVerify();
@@ -231,13 +243,13 @@ public:
 
    virtual const char *CurrentStatus();
 
-   virtual int Read(void *buf,int size) = 0;
+   virtual int Read(Buffer *buf,int size) = 0;
    virtual int Write(const void *buf,int size) = 0;
    virtual int Buffered();
    virtual int StoreStatus() = 0;
    virtual bool IOReady();
-   off_t GetPos() { return pos; }
-   off_t GetRealPos() { return real_pos<0?pos:real_pos; }
+   off_t GetPos() const { return pos; }
+   off_t GetRealPos() const { return real_pos<0?pos:real_pos; }
    void SeekReal() { pos=GetRealPos(); }
    void RereadManual() { norest_manual=true; }
 
@@ -253,18 +265,18 @@ public:
    bool IsBetterThan(const FileAccess *fa) const;
 
    void Init();
-   FileAccess() { Init(); }
+   FileAccess() : all_fa_node(this) { Init(); }
    FileAccess(const FileAccess *);
-   virtual ~FileAccess();
 
-   void	 DontSleep() { try_time=0; }
+   void	 DontSleep() { reconnect_timer.Stop(); }
 
    bool	 IsClosed() { return mode==CLOSED; }
    bool	 IsOpen() { return !IsClosed(); }
    int	 OpenMode() { return mode; }
 
    virtual int  IsConnected() const; // level of connection (0 - not connected).
-   virtual void Disconnect();
+   void Disconnect(const char *dc=0) { last_disconnect_cause.set(dc); DisconnectLL(); }
+   virtual void DisconnectLL() {}
    virtual void UseCache(bool);
    virtual bool NeedSizeDateBeforehand();
 
@@ -297,7 +309,7 @@ public:
    virtual DirList *MakeDirList(ArgV *a);
    virtual FileSet *ParseLongList(const char *buf,int len,int *err=0) const { return 0; }
 
-   static bool NotSerious(int err);
+   static bool NotSerious(int err) { return temporary_network_error(err); }
 
    const char *GetNewLocation() { return location; }
    const char *GetSuggestedFileName() { return suggested_filename; }
@@ -310,8 +322,7 @@ public:
    typedef FileAccess *SessionCreator();
    class Protocol
    {
-      static Protocol *chain;
-      Protocol *next;
+      static xmap<Protocol*> proto_by_name;
       const char *proto;
       SessionCreator *New;
 
@@ -344,9 +355,10 @@ public:
    int GetPriority() const { return priority; }
 
    // not pretty (FIXME)
-   int GetRetries() { return retries; }
+   int GetRetries() const { return retries; }
    void SetRetries(int r) { retries=r; }
-   time_t GetTryTime() { return try_time; }
+   int GetMaxRetries() const { return max_retries; }
+   time_t GetTryTime() const { return reconnect_timer.GetStartTime(); }
    void SetTryTime(time_t t);
 
    const char *GetLogContext() { return hostname; }
@@ -404,7 +416,6 @@ protected:
 
 public:
    FileAccessRefC(const FileAccessRef& p) { ref=&p; }
-   ~FileAccessRefC() { close(); }
    const FileAccessRef& operator=(const FileAccessRef& p) { close(); ref=&p; return p; }
    operator const FileAccess*() const { return *ref; }
    FileAccess *operator->() const { return ref->operator->(); }
@@ -435,7 +446,7 @@ protected:
 
    bool use_cache;
 
-   ~FileAccessOperation() { if(session) session->Close(); }
+   void PrepareToDie() { if(session) session->Close(); }
 
 public:
    FileAccessOperation(FileAccess *s) : session(s), done(false), use_cache(true) {}
@@ -464,10 +475,10 @@ protected:
    bool follow_symlinks;
 
    void PrepareToDie();
+   ~ListInfo();
 
 public:
    ListInfo(FileAccess *session,const char *path);
-   ~ListInfo();
 
    void SetExclude(const char *p,const PatternSet *e) { exclude_prefix=p; exclude=e; }
 
@@ -500,10 +511,10 @@ protected:
    Ref<Buffer> buf;
    Ref<ArgV> args;
    bool color;
+   ~DirList();
 
 public:
    DirList(FileAccess *s,ArgV *a);
-   ~DirList();
 
    virtual int Do() = 0;
    virtual const char *Status() = 0;
@@ -516,4 +527,36 @@ public:
    void UseColor(bool c=true) { color=c; }
 };
 
+class UploadState
+{
+   time_t try_time;
+   off_t pos_watermark;
+   int retries;
+public:
+   UploadState() : try_time(NO_DATE), pos_watermark(0),retries(-1) {}
+   void Clear() {
+      try_time=NO_DATE;
+      pos_watermark=0;
+      retries=-1;
+   }
+   void Save(const FileAccess *session) {
+      try_time=session->GetTryTime();
+      retries=session->GetRetries();
+      off_t pos=session->GetRealPos();
+      int max_retries=session->GetMaxRetries();
+      if(max_retries>0 && retries>=max_retries)
+	 pos=0;
+      if(pos_watermark<pos) {
+	 pos_watermark=pos;
+	 retries=-1;
+      }
+   }
+   void Restore(const FileAccessRef& session) {
+      if(try_time!=NO_DATE)
+	 session->SetTryTime(try_time);
+      if(retries>=0)
+	 session->SetRetries(retries+1);
+   }
+};
+
 #endif /* FILEACCESS_H */
