@@ -1051,7 +1051,8 @@ void FileCopyPeerFA::OpenSession()
    current->Timeout(0);	// mark it MOVED.
    if(mode==GET)
    {
-      if(size!=NO_SIZE && size!=NO_SIZE_YET && seek_pos>=size && !ascii)
+      if(size!=NO_SIZE && size!=NO_SIZE_YET && !ascii
+      && (seek_pos>size || (seek_pos==size && size>0)))
       {
       past_eof:
 	 debug((10,"copy src: seek past eof (seek_pos=%lld, size=%lld)\n",
@@ -1124,7 +1125,7 @@ void FileCopyPeerFA::WantSize()
    struct stat st;
    if(!strcmp(session->GetProto(),"file")
    && stat(dir_file(session->GetCwd(),file),&st)!=-1)
-      SetSize(st.st_size);
+      SetSize(S_ISREG(st.st_mode)?st.st_size:NO_SIZE);
    else
       super::WantSize();
 }
@@ -1516,9 +1517,15 @@ int FileCopyPeerFDStream::Do()
 	 {
 	    const char *new_name=dir_file(dirname(stream->full_name),suggested_filename);
 	    struct stat st;
-	    if((lstat(new_name,&st)==-1 && errno==ENOENT) || ResMgr::QueryBool("xfer:clobber",0)) {
+	    if(temp_file || (lstat(new_name,&st)==-1 && errno==ENOENT) || ResMgr::QueryBool("xfer:clobber",0)) {
 	       debug((5,"copy: renaming `%s' to `%s'\n",stream->full_name.get(),suggested_filename.get()));
-	       if(rename(stream->full_name,new_name)==-1) {
+	       int res=rename(stream->full_name,new_name);
+	       if(res==-1 && errno==EIO) {
+		  // FUSE with HadoopFS workaround
+		  unlink(new_name);
+		  res=rename(stream->full_name,new_name);
+	       }
+	       if(res==-1) {
 		  const char *err=xstring::format("rename(%s, %s): %s\n",stream->full_name.get(),new_name,strerror(errno));
 		  if(temp_file)
 		     SetError(err);
@@ -1671,7 +1678,7 @@ int FileCopyPeerFDStream::Get_LL(int len)
       else
       {
 	 SetDate(st.st_mtime);
-	 SetSize(st.st_size);
+	 SetSize(S_ISREG(st.st_mode)?st.st_size:NO_SIZE);
 #ifndef NATIVE_CRLF
 	 if(ascii)
 	    SetSize(NO_SIZE);
@@ -1834,15 +1841,15 @@ FgData *FileCopyPeerFDStream::GetFgData(bool fg)
 void FileCopyPeerFDStream::WantSize()
 {
    struct stat st;
-   st.st_size=NO_SIZE;
+   int res=-1;
 
    if(stream->fd!=-1)
-      fstat(stream->fd,&st);
+      res=fstat(stream->fd,&st);
    else if(stream->full_name)
-      stat(stream->full_name,&st);
+      res=stat(stream->full_name,&st);
 
-   if(st.st_size!=NO_SIZE)
-      SetSize(st.st_size);
+   if(res!=-1)
+      SetSize(S_ISREG(st.st_mode)?st.st_size:NO_SIZE);
    else
       super::WantSize();
 }
