@@ -54,6 +54,7 @@ protected:
    bool do_verify;
    bool removing;
    bool file_removed;
+   bool temp_file;
 
    bool done;
 
@@ -140,12 +141,9 @@ public:
    virtual const Ref<FDStream>& GetLocal() const { return Ref<FDStream>::null; }
 
    const char *GetSuggestedFileName() { return suggested_filename; }
-   void SetSuggestedFileName(const char *f)
-      {
-	 if(f && !suggested_filename)
-	    suggested_filename.set(f);
-      }
+   void SetSuggestedFileName(const char *f) { if(f) suggested_filename.set(f); }
    void AutoRename(bool yes=true) { auto_rename=yes; }
+   const char *UseTempFile(const char *);
 };
 
 class FileCopy : public SMTask
@@ -171,8 +169,8 @@ private:
 
    xstring_c error_text;
 
-   SMTaskRef<Speedometer> rate;
-   SMTaskRef<Speedometer> rate_for_eta;
+   Speedometer rate;
+   Speedometer rate_for_eta;
    int put_buf;
    off_t put_eof_pos;
 
@@ -199,23 +197,24 @@ public:
    off_t GetSize() const;
    int  GetPercentDone() const;
    const char *GetPercentDoneStr() const;
-   float GetRate() const;
-   const char *GetRateStr() const;
-   off_t GetBytesRemaining() const;
-   long GetETA() const { return GetETA(GetBytesRemaining()); }
-   long GetETA(off_t b) const;
-   const char *GetETAStr() const;
-   const char *GetETAStrSFromTime(time_t t) const { return rate_for_eta->GetETAStrSFromTime(t); }
+   float GetRate();
+   const char *GetRateStr();
+   off_t GetBytesRemaining();
+   long GetETA() { return GetETA(GetBytesRemaining()); }
+   long GetETA(off_t b);
+   const char *GetETAStr();
+   const char *GetETAStrSFromTime(time_t t) { return rate_for_eta.GetETAStrSFromTime(t); }
    const char *GetStatus();
    FgData *GetFgData(bool fg);
    pid_t GetProcGroup();
    void Kill(int sig);
    off_t GetBytesCount() { return bytes_count; }
    double GetTimeSpent();
+   double GetTransferRate() { return rate.Get(); }
 
    void SetDate(time_t t,int p=0) { get->SetDate(t,p); }
    void SetDate(const FileTimestamp &t) { SetDate(t.ts,t.ts_prec); }
-   void SetSize(off_t  s) { get->SetSize(s); }
+   void SetSize(off_t s) { get->SetSize(s); }
 
    bool SetContinue(bool new_cont) { return replace_value(cont,new_cont); }
 
@@ -290,17 +289,18 @@ class FileCopyPeerFA : public FileCopyPeer
    int Put_LL(const char *buf,int size);
    int PutEOF_LL();
 
+   // to read data in larger quantities, delay the read op
+   Timer get_ll_timer;
+   int get_delay;
+
    FileSet info;
 
    bool fxp;   // FXP (ftp<=>ftp copy) active
 
-   time_t try_time;
-   int retries;
-   off_t upload_watermark;
-
+   UploadState upload_state;
    int redirections;
 
-   Ref<FileVerificator> verify;
+   SMTaskRef<FileVerificator> verify;
 
 protected:
    void PrepareToDie();
@@ -360,8 +360,9 @@ class FileCopyPeerFDStream : public FileCopyPeer
 
    bool create_fg_data;
    bool need_seek;
+   bool close_when_done;
 
-   Ref<FileVerificator> verify;
+   SMTaskRef<FileVerificator> verify;
 
 public:
    void Init();
@@ -376,6 +377,7 @@ public:
 
    void DontCreateFgData() { create_fg_data=false; }
    void NeedSeek() { need_seek=true; }
+   void CloseWhenDone() { close_when_done=true; }
    void WantSize();
    void RemoveFile();
    void SetBase(off_t b) { seek_base=b; }
