@@ -298,6 +298,29 @@ void FileSet::SubtractSameType(const FileSet *set)
 	 Sub(i--);
    }
 }
+void FileSet::SubtractDirs(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(!files[i]->TypeIs(FileInfo::DIRECTORY))
+	 continue;
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && f->TypeIs(f->DIRECTORY))
+	 Sub(i--);
+   }
+}
+void FileSet::SubtractNotOlderDirs(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(!files[i]->TypeIs(FileInfo::DIRECTORY)
+      || !files[i]->Has(FileInfo::DATE))
+	 continue;
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && f->TypeIs(f->DIRECTORY) && f->NotOlderThan(files[i]->date))
+	 Sub(i--);
+   }
+}
 
 void FileSet::SubtractTimeCmp(bool (FileInfo::*cmp)(time_t) const,time_t t)
 {
@@ -364,6 +387,14 @@ void FileSet::ExcludeDots()
       }
    }
 }
+void FileSet::ExcludeCompound()
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(strchr(files[i]->name,'/'))
+	 Sub(i--);
+   }
+}
 
 void FileSet::ExcludeUnaccessible()
 {
@@ -763,9 +794,11 @@ drwxr-xr-x   4 lav      root         1024 Feb 22 15:32 lib
 lrwxrwxrwx   1 lav      root           33 Feb 14 17:45 ltconfig -> /usr/share/libtool/ltconfig
 NOTE: group may be missing.
 */
-FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
+FileInfo *FileInfo::parse_ls_line(const char *line_c,int line_len,const char *tz)
 {
-   char *line=alloca_strdup(line_c);
+   char *line=string_alloca(line_len+1);
+   memcpy(line,line_c,line_len);
+   line[line_len]=0;
    char *next=0;
    FileInfo *fi=0; /* don't instantiate until we at least have something */
 #define FIRST_TOKEN strtok_r(line," \t",&next)
@@ -794,7 +827,7 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    case('p'): // pipe
    case('s'): // sock
    case('D'): // Door
-      return 0;  // ignore
+      // ignore them
    default:
       ERR;
    }
@@ -921,8 +954,11 @@ int FileSet::Have() const
    return bits;
 }
 
-static bool fnmatch_dir(const char *pattern,const FileInfo *file)
+static int fnmatch_dir(const char *pattern,const FileInfo *file)
 {
+   bool inverted = (pattern[0]=='!');
+   if(inverted || (pattern[0]=='\\' && pattern[1]=='!'))
+      pattern++;
    const char *name=file->name;
    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
    {
@@ -930,7 +966,15 @@ static bool fnmatch_dir(const char *pattern,const FileInfo *file)
       strcat(n,"/");
       name=n;
    }
-   return fnmatch(pattern,name,FNM_PATHNAME|FNM_CASEFOLD);
+   int result=fnmatch(pattern,name,FNM_PATHNAME|FNM_CASEFOLD);
+   if(inverted)
+   {
+      if(result==0)
+	 result=FNM_NOMATCH;
+      else if(result==FNM_NOMATCH)
+	 result=0;
+   }
+   return result;
 }
 
 void FileSet::SortByPatternList(const char *list_c)
