@@ -93,7 +93,8 @@ public:
    void SetUser(const char *n);
    void SetGroup(const char *n);
    void LocalFile(const char *name, bool follow_symlinks);
-   static FileInfo *parse_ls_line(const char *line,const char *tz);
+   static FileInfo *parse_ls_line(const char *line,int line_len,const char *tz);
+   static FileInfo *parse_ls_line(const char *line,const char *tz) { return parse_ls_line(line,strlen(line),tz); }
 
    void SetMode(mode_t m) { mode=m; def(MODE); }
    void SetDate(time_t t,int prec) { date.set(t,prec); def(DATE); }
@@ -110,6 +111,7 @@ public:
    bool	 NotOlderThan(time_t t) const;
    bool	 NotNewerThan(time_t t) const;
    bool  SizeOutside(const Range *r) const;
+   bool	 TypeIs(type t) const { return (defined&TYPE) && filetype==t; }
 
    void	 SetAssociatedData(const void *d,int len) { data.nset((const char*)d,len); }
    const void *GetAssociatedData() const { return data; }
@@ -175,13 +177,17 @@ public:
    void  SubtractNotDirs();
    void  SubtractNotIn(const FileSet *);
    void  SubtractSameType(const FileSet *);
+   void  SubtractDirs(const FileSet *);
+   void  SubtractNotOlderDirs(const FileSet *);
    void  SubtractCurr();
    void  Sort(sort_e newsort, bool casefold=false, bool reverse=false);
    void  Unsort();
    void	 SortByPatternList(const char *list_c);
+   void	 ReverseSort();
 
    void	 Exclude(const char *prefix,const PatternSet *x);
    void	 ExcludeDots();
+   void	 ExcludeCompound();
    void	 ExcludeUnaccessible();
 
    void	 rewind() { ind=0; }
