@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ CDECL_END
 
 
 ResDecl	res_default_cls         ("cmd:cls-default",  "-F", FileSetOutput::ValidateArgv,ResMgr::NoClosure),
-	res_default_comp_cls    ("cmd:cls-completion-default", "-FB",FileSetOutput::ValidateArgv,ResMgr::NoClosure);
+	res_default_comp_cls    ("cmd:cls-completion-default", "-FBa",FileSetOutput::ValidateArgv,ResMgr::NoClosure);
 
 ResDecl res_time_style	("cmd:time-style", "%b %e  %Y|%b %e %H:%M", 0, ResMgr::NoClosure);
 
@@ -89,7 +89,6 @@ void FileSetOutput::print(FileSet &fs, const JobRef<OutputJob>& o) const
 	 if(f->filetype == FileInfo::DIRECTORY) mode[0] = 'd';
 	 else if(f->filetype == FileInfo::SYMLINK) mode[0] = 'l';
 	 else mode[0] = '-';
-	 strcat(mode, " ");
 
 	 c.add(mode, "");
       }
@@ -274,7 +273,6 @@ clsJob::clsJob(FA *s, ArgV *a, FileSetOutput *_opts, OutputJob *_output):
    done(0),
    use_cache(true),
    error(false),
-   num(1),
    state(INIT)
 {
    list_info=0;
@@ -376,6 +374,7 @@ int clsJob::Do()
 
 void clsJob::SuspendInternal()
 {
+   super::SuspendInternal();
    if(list_info)
       list_info->SuspendSlave();
    session->SuspendSlave();
@@ -386,6 +385,7 @@ void clsJob::ResumeInternal()
    if(list_info)
       list_info->ResumeSlave();
    session->ResumeSlave();
+   super::ResumeInternal();
 }
 
 void clsJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
