@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FtpListInfo.cc,v 1.66 2011/06/28 09:45:56 lav Exp $ */
-
 #include <config.h>
 #include "FtpListInfo.h"
 #include "FileSet.h"
@@ -727,10 +725,13 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
    bool type_known=false;
    int perms=-1;
 
-   /* NcFTPd does not put a semicolon after last fact, workaround it. */
-   if(!strstr(line,"; "))
-   {
-      char *space=strchr(line,' ');
+   char *space=strstr(line,"; ");
+   if(space) {
+      name=space+2;
+      *space=0;
+   } else {
+      /* NcFTPd does not put a semicolon after last fact, workaround it. */
+      space=strchr(line,' ');
       if(!space)
 	 ERR;
       name=space+1;
@@ -739,11 +740,6 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
 
    for(char *tok=strtok(line,";"); tok; tok=strtok(0,";"))
    {
-      if(tok[0]==' ')
-      {
-	 name=tok+1;
-	 break;
-      }
       if(!strcasecmp(tok,"Type=cdir")
       || !strcasecmp(tok,"Type=pdir")
       || !strcasecmp(tok,"Type=dir"))
