@@ -79,12 +79,14 @@ class Http : public NetAccess
 
    Ref<Connection> conn;
 
+   static void AppendHostEncoded(xstring&,const char *);
    void SendMethod(const char *,const char *);
    const char *last_method;
    enum { HTTP_NONE=0, HTTP_POST, HTTP_MOVE, HTTP_COPY, HTTP_PROPFIND } special;
    xstring special_data;
    void DirFile(xstring& path,const xstring& ecwd,const xstring& efile) const;
    void SendAuth();
+   void SendProxyAuth();
    void SendCacheControl();
    void SendBasicAuth(const char *tag,const char *auth);
    void SendBasicAuth(const char *tag,const char *u,const char *p);
