@@ -261,12 +261,12 @@ void  MirrorJob::HandleFile(FileInfo *file)
    target_name_rel=alloca_strdup(target_name_rel);
 
    FileInfo::type filetype=FileInfo::NORMAL;
-   if(file->defined&file->TYPE)
+   if(file->Has(file->TYPE))
       filetype=file->filetype;
    else
    {
       FileInfo *target=target_set->FindByName(file->name);
-      if(target && (target->defined&target->TYPE))
+      if(target && target->Has(target->TYPE))
 	 filetype=target->filetype;
    }
 
@@ -277,7 +277,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 bool remove_target=false;
 	 bool cont_this=false;
 	 bool use_pget=(pget_n>1) && target_is_local;
-	 if((file->defined&file->SIZE) && file->size<pget_minchunk*2)
+	 if(file->Has(file->SIZE) && file->size<pget_minchunk*2)
 	    use_pget=false;
 	 if(target_is_local)
 	 {
@@ -288,8 +288,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       if(old)
 		  goto skip;  // file has appeared after mirror start
 	       old=old_files_set->FindByName(file->name);
-	       if(old && ((old->defined&old->SIZE && old->size!=st.st_size)
-			||(old->defined&old->DATE && old->date!=st.st_mtime)))
+	       if(old && ((old->Has(old->SIZE) && old->size!=st.st_size)
+			||(old->Has(old->DATE) && old->date!=st.st_mtime)))
 		  goto skip;  // the file has changed after mirror start
 	    }
 	 }
@@ -297,11 +297,11 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 if(old)
 	 {
 	    if((flags&CONTINUE)
-	    && (old->defined&file->TYPE) && old->filetype==old->NORMAL
+	    && old->Has(file->TYPE) && old->filetype==old->NORMAL
 	    && (flags&IGNORE_TIME ||
-	    	((file->defined&file->DATE) && (old->defined&old->DATE)
+	    	(file->Has(file->DATE) && old->Has(old->DATE)
 	    	&& file->date + file->date.ts_prec < old->date - old->date.ts_prec))
-	    && (file->defined&file->SIZE) && (old->defined&old->SIZE)
+	    && file->Has(file->SIZE) && old->Has(old->SIZE)
 	    && file->size >= old->size)
 	    {
 	       cont_this=true;
@@ -317,8 +317,12 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    }
 	    else if(!to_rm_mismatched->FindByName(file->name))
 	    {
-	       Report(_("Removing old file `%s'"),target_name_rel);
-	       remove_target=true;
+	       if(!FlagSet(OVERWRITE)) {
+		  remove_target=true;
+		  Report(_("Removing old file `%s'"),target_name_rel);
+	       } else {
+		  Report(_("Overwriting old file `%s'"),target_name_rel);
+	       }
 	       stats.mod_files++;
 	    }
 	    else
@@ -380,9 +384,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 if(FlagSet(ASCII))
 	    c->Ascii();
 	 CopyJob *cp=(use_pget ? new pgetJob(c,file->name,pget_n) : new CopyJob(c,file->name,"mirror"));
-	 if(file->defined&file->DATE)
+	 if(file->Has(file->DATE))
 	    cp->SetDate(file->date);
-	 if(file->defined&file->SIZE)
+	 if(file->Has(file->SIZE))
 	    cp->SetSize(file->size);
 	 TransferStarted(cp);
 	 cp->cmdline.vset("\\transfer `",source_name_rel,"'",NULL);
@@ -392,7 +396,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
       }
       case(FileInfo::DIRECTORY):
       {
-	 if(flags&NO_RECURSION)
+	 if(recursion_mode==RECURSION_NEVER || FlagSet(NO_RECURSION))
 	    goto skip;
 
 	 bool create_target_dir=true;
@@ -407,7 +411,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       goto skip;
 	    }
 	 }
-	 else if(old->defined&old->TYPE && old->filetype==old->DIRECTORY)
+	 else if(old->TypeIs(old->DIRECTORY))
 	 {
 	    create_target_dir=false;
 	 }
@@ -460,6 +464,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->skip_noaccess=skip_noaccess;
 	 mj->create_target_dir=create_target_dir;
 	 mj->no_target_dir=no_target_dir;
+	 mj->recursion_mode=recursion_mode;
 
 	 mj->script=script;
 	 mj->script_needs_closing=false;
@@ -528,7 +533,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       goto skip;
 	 }
 
-	 if(file->defined&file->SYMLINK)
+	 if(file->Has(file->SYMLINK))
 	 {
 	    struct stat st;
 	    if(lstat(target_name,&st)!=-1)
@@ -577,7 +582,7 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
    int ignore=0;
    if(flags&ONLY_NEWER)
       ignore|=FileInfo::IGNORE_SIZE_IF_OLDER|FileInfo::IGNORE_DATE_IF_OLDER;
-   if(strcmp(target_session->GetProto(),"file"))
+   if(!FlagSet(UPLOAD_OLDER) && strcmp(target_session->GetProto(),"file"))
       ignore|=FileInfo::IGNORE_DATE_IF_OLDER;
    if(flags&IGNORE_TIME)
       ignore|=FileInfo::DATE;
@@ -600,8 +605,19 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
       to_mkdir->SubtractAny(dest);
    }
 
-   if(flags&NO_RECURSION)
+   switch(recursion_mode) {
+   case RECURSION_NEVER:
       to_transfer->SubtractDirs();
+      break;
+   case RECURSION_MISSING:
+      to_transfer->SubtractDirs(dest);
+      break;
+   case RECURSION_NEWER:
+      to_transfer->SubtractNotOlderDirs(dest);
+      break;
+   case RECURSION_ALWAYS:
+      break;
+   }
 
    if(skip_noaccess)
       to_transfer->ExcludeUnaccessible();
@@ -660,9 +676,9 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 	    char *loc=alloca_strdup(loc_c);
 	    ParsedURL u(loc,true);
 
+	    bool is_file=(last_char(loc)!='/');
 	    if(!u.proto)
 	    {
-	       bool is_file=(last_char(loc)!='/');
 	       FileAccess::Path new_cwd(session->GetNewCwd());
 	       new_cwd.Change(0,is_file,loc);
 	       session->PathVerify(new_cwd);
@@ -671,7 +687,8 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 	    }
 	    session->Close(); // loc_c is no longer valid.
 	    session=FA::New(&u);
-	    session->Chdir(u.path);
+	    FileAccess::Path new_cwd(u.path,is_file,url::path_ptr(loc));
+	    session->PathVerify(new_cwd);
 	    return;
 	 }
       }
@@ -773,7 +790,7 @@ int   MirrorJob::Do()
       if(source_session->IsOpen())
 	 return m;
 
-      source_dir.set(source_session->GetCwd());
+      source_dir.set(source_session->GetCwd().GetDirectory());
 
    pre_MAKE_TARGET_DIR:
    {
@@ -856,7 +873,7 @@ int   MirrorJob::Do()
 	 return m;
       create_target_dir=false;
 
-      target_dir.set(target_session->GetCwd());
+      target_dir.set(target_session->GetCwd().GetDirectory());
 
    pre_GETTING_LIST_INFO:
       set_state(GETTING_LIST_INFO);
@@ -1067,9 +1084,9 @@ int   MirrorJob::Do()
 	 if(script)
 	 {
 	    ArgV args("rm");
-	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
+	    if(file->TypeIs(file->DIRECTORY))
 	    {
-	       if(FlagSet(NO_RECURSION) && !FlagSet(SCAN_ALL_FIRST))
+	       if(recursion_mode==RECURSION_NEVER)
 		  args.setarg(0,"rmdir");
 	       else
 		  args.Append("-r");
@@ -1086,9 +1103,9 @@ int   MirrorJob::Do()
 	    rmJob *j=new rmJob(target_session->Clone(),args);
 	    j->cmdline.set_allocated(args->Combine());
 	    JobStarted(j);
-	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
+	    if(file->TypeIs(file->DIRECTORY))
 	    {
-	       if(FlagSet(NO_RECURSION) && !FlagSet(SCAN_ALL_FIRST))
+	       if(recursion_mode==RECURSION_NEVER)
 	       {
 		  args->setarg(0,"rmdir");
 		  j->Rmdir();
@@ -1126,12 +1143,12 @@ int   MirrorJob::Do()
 	 if(!file)
 	    goto pre_FINISHING_FIX_LOCAL;
 	 to_transfer->next();
-	 if((file->defined&file->TYPE) && file->filetype==file->SYMLINK)
+	 if(file->TypeIs(file->SYMLINK))
 	    continue;
-	 if(!(file->defined&file->MODE))
+	 if(!file->Has(file->MODE))
 	    continue;
 	 mode_t mode_mask=get_mode_mask();
-	 mode_t def_mode=(file->filetype==file->DIRECTORY?0775:0664)&~mode_mask;
+	 mode_t def_mode=(file->TypeIs(file->DIRECTORY)?0775:0664)&~mode_mask;
 	 if(target_is_local && file->mode==def_mode)
 	 {
 	    struct stat st;
@@ -1549,6 +1566,27 @@ const char *MirrorJob::AddPattern(Ref<PatternSet>& exclude,char opt,const char *
    return NULL; // no error
 }
 
+const char *MirrorJob::SetRecursionMode(const char *m)
+{
+   struct { const char name[8]; recursion_mode_t mode; } map[]={
+      {"always", RECURSION_ALWAYS},
+      {"never",  RECURSION_NEVER},
+      {"missing",RECURSION_MISSING},
+      {"newer",  RECURSION_NEWER},
+   };
+   unsigned i;
+   for(i=0; i<sizeof(map)/sizeof(map[0]); i++) {
+      if(!strcasecmp(m,map[i].name)) {
+	 recursion_mode=map[i].mode;
+	 return 0;
+      }
+   }
+   xstring list(map[0].name);
+   for(i=1; i<sizeof(map)/sizeof(map[0]); i++)
+      list.append(", ").append(map[i].name);
+   return xstring::format(_("%s must be one of: %s"),"--recursion",list.get());
+}
+
 CMD(mirror)
 {
 #define args (parent->args)
@@ -1579,6 +1617,10 @@ CMD(mirror)
       OPT_DEPTH_FIRST,
       OPT_ASCII,
       OPT_SCAN_ALL_FIRST,
+      OPT_OVERWRITE,
+      OPT_NO_OVERWRITE,
+      OPT_RECURSION,
+      OPT_UPLOAD_OLDER,
    };
    static const struct option mirror_opts[]=
    {
@@ -1627,6 +1669,10 @@ CMD(mirror)
       {"target-directory",required_argument,0,'O'},
       {"destination-directory",required_argument,0,'O'},
       {"scan-all-first",no_argument,0,OPT_SCAN_ALL_FIRST},
+      {"overwrite",no_argument,0,OPT_OVERWRITE},
+      {"no-overwrite",no_argument,0,OPT_NO_OVERWRITE},
+      {"recursion",required_argument,0,OPT_RECURSION},
+      {"upload-older",no_argument,0,OPT_UPLOAD_OLDER},
       {0}
    };
 
@@ -1652,6 +1698,7 @@ CMD(mirror)
    bool	 no_empty_dirs=ResMgr::QueryBool("mirror:no-empty-dirs",0);
    const char *script_file=0;
    const char *on_change=0;
+   const char *recursion_mode=0;
 
    Ref<PatternSet> exclude;
 
@@ -1659,6 +1706,8 @@ CMD(mirror)
       flags|=MirrorJob::NO_PERMS;
    if(ResMgr::QueryBool("mirror:dereference",0))
       flags|=MirrorJob::RETR_SYMLINKS;
+   if(ResMgr::QueryBool("mirror:overwrite",0))
+      flags|=MirrorJob::OVERWRITE;
 
    const char *source_dir=NULL;
    const char *target_dir=NULL;
@@ -1681,8 +1730,7 @@ CMD(mirror)
 	 flags|=MirrorJob::ALLOW_SUID|MirrorJob::ALLOW_CHOWN|MirrorJob::NO_UMASK;
 	 break;
       case('r'):
-	 flags|=MirrorJob::NO_RECURSION;
-	 flags&=~MirrorJob::SCAN_ALL_FIRST;
+	 recursion_mode="never";
 	 break;
       case('n'):
 	 flags|=MirrorJob::ONLY_NEWER;
@@ -1733,8 +1781,7 @@ CMD(mirror)
       case('f'):
       {
 	 // mirror for a single file (or glob pattern).
-	 flags|=MirrorJob::NO_RECURSION;
-	 flags&=~MirrorJob::SCAN_ALL_FIRST;
+	 recursion_mode="never";
 	 MirrorJob::AddPattern(exclude,'I',basename_ptr(optarg));
 	 source_dir=dirname(optarg);
 	 source_dir=alloca_strdup(source_dir); // save the temp string
@@ -1826,6 +1873,18 @@ CMD(mirror)
       case(OPT_ASCII):
 	 flags|=MirrorJob::ASCII|MirrorJob::IGNORE_SIZE;
 	 break;
+      case(OPT_OVERWRITE):
+	 flags|=MirrorJob::OVERWRITE;
+	 break;
+      case(OPT_NO_OVERWRITE):
+	 flags&=~MirrorJob::OVERWRITE;
+	 break;
+      case(OPT_RECURSION):
+	 recursion_mode=optarg;
+	 break;
+      case(OPT_UPLOAD_OLDER):
+	 flags|=MirrorJob::UPLOAD_OLDER;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
@@ -1833,7 +1892,7 @@ CMD(mirror)
       }
    }
 
-   if(exclude && !(flags&MirrorJob::NO_RECURSION))
+   if(exclude && xstrcasecmp(recursion_mode,"never"))
    {
       /* Users usually don't want to exclude all directories when recursing */
       if(exclude->GetFirstType()==PatternSet::INCLUDE)
@@ -1940,7 +1999,7 @@ CMD(mirror)
    if(no_empty_dirs)
       flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
 
-   Ref<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
+   JobRef<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
    j->SetFlags(flags,1);
    j->SetVerbose(verbose);
    j->SetExclude(exclude.borrow());
@@ -1965,13 +2024,19 @@ CMD(mirror)
    if(use_pget>1 && !(flags&MirrorJob::ASCII))
       j->SetPGet(use_pget);
 
+   if(recursion_mode) {
+      const char *err=j->SetRecursionMode(recursion_mode);
+      if(err) {
+	 eprintf("%s: %s\n",args->a0(),err);
+	 return 0;
+      }
+   }
    if(script_file)
    {
       const char *err=j->SetScriptFile(script_file);
       if(err)
       {
 	 eprintf("%s: %s\n",args->a0(),err);
-	 j->DeleteLater();
 	 return 0;
       }
    }
@@ -1991,9 +2056,37 @@ CMD(mirror)
 }
 
 #include "modconfig.h"
-#ifdef MODULE_CMD_MIRROR
-void module_init()
-{
-   CmdExec::RegisterCommand("mirror",cmd_mirror);
-}
+#ifndef MODULE_CMD_MIRROR
+# define module_init cmd_mirror_module_init
 #endif
+CDECL void module_init()
+{
+   CmdExec::RegisterCommand("mirror",cmd_mirror,0,
+	 N_("\nMirror specified remote directory to local directory\n\n"
+	 " -c, --continue         continue a mirror job if possible\n"
+	 " -e, --delete           delete files not present at remote site\n"
+	 "     --delete-first     delete old files before transferring new ones\n"
+	 " -s, --allow-suid       set suid/sgid bits according to remote site\n"
+	 "     --allow-chown      try to set owner and group on files\n"
+	 "     --ignore-time      ignore time when deciding whether to download\n"
+	 " -n, --only-newer       download only newer files (-c won't work)\n"
+	 " -r, --no-recursion     don't go to subdirectories\n"
+	 " -p, --no-perms         don't set file permissions\n"
+	 "     --no-umask         don't apply umask to file modes\n"
+	 " -R, --reverse          reverse mirror (put files)\n"
+	 " -L, --dereference      download symbolic links as files\n"
+	 " -N, --newer-than=SPEC  download only files newer than specified time\n"
+	 " -P, --parallel[=N]     download N files in parallel\n"
+	 " -i RX, --include RX    include matching files\n"
+	 " -x RX, --exclude RX    exclude matching files\n"
+	 "                        RX is extended regular expression\n"
+	 " -v, --verbose[=N]      verbose operation\n"
+	 "     --log=FILE         write lftp commands being executed to FILE\n"
+	 "     --script=FILE      write lftp commands to FILE, but don't execute them\n"
+	 "     --just-print, --dry-run    same as --script=-\n"
+	 "\n"
+	 "When using -R, the first directory is local and the second is remote.\n"
+	 "If the second directory is omitted, basename of first directory is used.\n"
+	 "If both directories are omitted, current local and remote directories are used.\n")
+   );
+}
