@@ -28,6 +28,15 @@
 
 class MirrorJob : public Job
 {
+public:
+   enum recursion_mode_t {
+      RECURSION_ALWAYS,
+      RECURSION_NEVER,
+      RECURSION_MISSING,
+      RECURSION_NEWER,
+   };
+
+private:
    enum state_t
    {
       INITIAL_STATE,
@@ -39,6 +48,8 @@ class MirrorJob : public Job
       TARGET_REMOVE_OLD,
       TARGET_REMOVE_OLD_FIRST,
       TARGET_CHMOD,
+      TARGET_MKDIR,
+      SOURCE_REMOVING_SAME,
       FINISHING,
       LAST_EXEC,
       DONE
@@ -54,7 +65,10 @@ class MirrorJob : public Job
    long long bytes_to_transfer;
    Ref<FileSet> target_set;
    Ref<FileSet> source_set;
+   Ref<FileSet> target_set_recursive;
+   Ref<FileSet> source_set_recursive;
    Ref<FileSet> to_transfer;
+   Ref<FileSet> to_mkdir;
    Ref<FileSet> same;
    Ref<FileSet> to_rm;
    Ref<FileSet> to_rm_mismatched;
@@ -108,6 +122,7 @@ class MirrorJob : public Job
    int	 &transfer_count;
 
    int	 flags;
+   recursion_mode_t recursion_mode;
    int	 max_error_count;
 
    Ref<PatternSet> my_exclude;
@@ -188,8 +203,10 @@ public:
       NO_EMPTY_DIRS=1<<16,
       DEPTH_FIRST=1<<17,
       ASCII=1<<18,
+      SCAN_ALL_FIRST=1<<19,
+      OVERWRITE=1<<20,
+      UPLOAD_OLDER=1<<21,
    };
-
    void SetFlags(int f,bool v)
    {
       if(v)
@@ -235,6 +252,7 @@ public:
    void Fg();
    void Bg();
 
+   const char *SetRecursionMode(const char *r);
    const char *SetScriptFile(const char *n);
    void	 ScriptOnly(bool yes=true)
       {
