@@ -210,7 +210,7 @@ int   Resolver::Do()
 	 LogNote(4,_("Resolving host address..."));
 	 buf=new IOBuffer(IOBuffer::GET);
 	 DoGethostbyname();
-	 if(deleting)
+	 if(Deleted())
 	    return MOVED;
       }
    }
@@ -490,7 +490,7 @@ void Resolver::LookupSRV_RR()
       if(!use_fork)
       {
 	 Schedule();
-	 if(deleting)
+	 if(Deleted())
 	    return;
       }
       time(&try_time);
@@ -679,6 +679,9 @@ void Resolver::LookupOne(const char *name)
       name=proto_delim+1;
    }
 
+   name=xidna_to_ascii(name);
+   name=alloca_strdup(name);
+
    ParseOrder(order,af_order);
 
    int retries=0;
@@ -688,7 +691,7 @@ void Resolver::LookupOne(const char *name)
       if(!use_fork)
       {
 	 Schedule();
-	 if(deleting)
+	 if(Deleted())
 	    return;
       }
 
@@ -875,12 +878,12 @@ void Resolver::DoGethostbyname()
    if(service && !portname && !isdigit((unsigned char)hostname[0]))
       LookupSRV_RR();
 
-   if(!use_fork && deleting)
+   if(!use_fork && Deleted())
       return;
 
    LookupOne(hostname);
 
-   if(!use_fork && deleting)
+   if(!use_fork && Deleted())
       return;
 
    if(addr.count()==0)
