@@ -56,6 +56,43 @@ public:
    void set_range(int from,int to,bool value);
 };
 
+class TorrentBuild : public SMTask, public ProtoLog
+{
+   xstring_c top_path;
+   xstring name;
+   FileSet files;
+   StringSet dirs_to_scan;
+   bool done;
+   Ref<Error> error;
+
+   Ref<DirectedBuffer> translate;
+   const char *lc_to_utf8(const char *s);
+
+   friend class Torrent;
+   Ref<BeNode> info;
+   xstring pieces;
+   off_t total_length;
+   unsigned piece_length;
+
+   const char *CurrPath() const { return dirs_to_scan[0]; }
+   void NextDir() { dirs_to_scan.Remove(0); }
+   void QueueDir(const char *dir) { dirs_to_scan.Append(dir); }
+   void AddFile(const char *path,struct stat *st);
+   void Finish();
+
+public:
+   TorrentBuild(const char *top);
+   int Do();
+   bool Done() const { return done || error; }
+   bool Failed() const { return error; }
+   const char *ErrorText() const { return error->Text(); }
+   BeNode *GetFilesNode() const { return info->lookup("files"); }
+   void SetPiece(unsigned p,const xstring& sha1);
+   const xstring& GetMetadata();
+   const char *GetBaseDirectory() const { return dirname(top_path); }
+   const xstring& Status() const;
+};
+
 class TorrentPiece
 {
    unsigned sources_count;	    // how many peers have the piece
@@ -193,6 +230,10 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    bool is_private;
    bool validating;
    bool force_valid;
+   bool build_md;
+   bool stop_if_complete;
+   bool stop_if_known;
+   bool md_saved;
    unsigned validate_index;
    Ref<Error> invalid_cause;
 
@@ -232,8 +273,8 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    static const SMTaskRef<TorrentListener>& GetUDPSocket(const sockaddr_u& a) { return GetUDPSocket(a.family()); }
 
    static Torrent *FindTorrent(const xstring& info_hash) { return torrents.lookup(info_hash); }
-   static void AddTorrent(Torrent *t) { torrents.add(t->GetInfoHash(),t); }
-   static void RemoveTorrent(Torrent *t) { torrents.remove(t->GetInfoHash()); }
+   static void AddTorrent(Torrent *t);
+   static void RemoveTorrent(Torrent *t);
    static int GetTorrentsCount() { return torrents.count(); }
    static void Dispatch(const xstring& info_hash,int s,const sockaddr_u *remote_addr,IOBuffer *recv_buf);
    static void DispatchUDP(const char *buf,int len,const sockaddr_u& src);
@@ -243,14 +284,20 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    void FetchMetadataFromURL(const char *url);
    void StartMetadataDownload();
    void MetadataDownloaded();
-   void SetMetadata(const xstring& md);
+   bool SetMetadata(const xstring& md);
    void ParseMagnet(const char *p);
    const char *GetMetadataPath() const;
-   void SaveMetadata() const;
+   bool SaveMetadata() const;
    bool LoadMetadata(const char *path);
 
+   void Startup();
+
+   void SetTotalLength(off_t);
+   void StartValidating();
+
    xstring_c metainfo_url;
    SMTaskRef<FileCopy> metainfo_copy;
+   SMTaskRef<TorrentBuild> building;
    Ref<BeNode> metainfo_tree;
    BeNode *info;
    xstring metadata;
@@ -279,7 +326,6 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    void AnnounceDHT();
    void DenounceDHT();
 
-   bool single_file;
    unsigned piece_length;
    unsigned last_piece_length;
    unsigned total_pieces;
@@ -398,6 +444,7 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    void PeerBytesGot(int b) { PeerBytesUsed(b,RateLimit::GET); }
 
    static void BlackListPeer(const TorrentPeer *peer,const char *timeout);
+   static bool BlackListed(const TorrentPeer *peer);
    TorrentPeer *FindPeerById(const xstring& p_id);
 
 public:
@@ -453,6 +500,11 @@ public:
 
    void ForceValid() { force_valid=true; }
    bool IsValidating() const { return validating; }
+   void Share() { build_md=true; }
+   bool IsSharing() const { return build_md; }
+
+   void StopIfComplete() { stop_if_complete=true; }
+   void StopIfKnown() { stop_if_known=stop_if_complete=true; }
 
    static int GetPort();
    static int GetPortIPv4() { return listener?listener->GetPort():0; }
@@ -786,7 +838,7 @@ public:
 	    return res;
 	 }
       void ComputeLength() { Packet::ComputeLength(); length++; if(data) length+=data->ComputeLength(); length+=appendix.length(); }
-      void Pack(SMTaskRef<IOBuffer>& b) { Packet::Pack(b); b->PackUINT8(code); if(data) data->Pack(b); }
+      void Pack(SMTaskRef<IOBuffer>& b) { Packet::Pack(b); b->PackUINT8(code); if(data) data->Pack(b); b->Put(appendix); }
       void SetAppendix(const char *s,int len) { appendix.nset(s,len); length+=len; }
    };
 
