@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: history.cc,v 1.23 2011/03/25 14:35:43 lav Exp $ */
-
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -38,7 +36,7 @@ History::History()
    stamp=0;
    fd=-1;
    modified=false;
-   const char *home=get_lftp_home();
+   const char *home=get_lftp_data_dir();
    if(home)
       file.vset(home,"/cwd_history",NULL);
 }
@@ -142,7 +140,7 @@ void History::Set(const FileAccess *s,const FileAccess::Path &cwd)
    if(!cwd.url)
    {
       res.append_url_encoded(cwd,URL_PATH_UNSAFE);
-      if(!cwd.is_file && url::dir_needs_trailing_slash(s->GetProto()))
+      if(!cwd.is_file && url::dir_needs_trailing_slash(s->GetProto()) && res.last_char()!='/')
 	 res.append('/');
    }
    else
