@@ -31,6 +31,9 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <pwd.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -882,14 +885,34 @@ const char *memrchr(const char *buf,char c,size_t len)
    return 0;
 }
 
-const xstring& shell_encode(const char *string)
+bool is_shell_special(char c)
+{
+   switch (c)
+   {
+   case '\'':
+   case '(': case ')':
+   case '!': case '{': case '}':		/* reserved words */
+   case '^':
+   case '$': case '`':			/* expansion chars */
+   case '*': case '[': case '?': case ']':	/* globbing chars */
+   case ' ': case '\t': case '\n':		/* IFS white space */
+   case '"': case '\\':		/* quoting chars */
+   case '|': case '&': case ';':		/* shell metacharacters */
+   case '<': case '>':
+   case '#':				/* comment char */
+      return true;
+   }
+   return false;
+}
+
+const xstring& shell_encode(const char *string,int len)
 {
    if(!string)
       return xstring::null;
 
    static xstring result;
 
-   result.get_space(2 + 2 * strlen (string));
+   result.get_space(2 + 2 * len);
    char *r = result.get_non_const();
 
    if(string[0]=='-' || string[0]=='~')
@@ -901,26 +924,9 @@ const xstring& shell_encode(const char *string)
    int c;
    for (const char *s = string; s && (c = *s); s++)
    {
-      switch (c)
-      {
-      case '\'':
-      case '(': case ')':
-      case '!': case '{': case '}':		/* reserved words */
-      case '^':
-      case '$': case '`':			/* expansion chars */
-      case '*': case '[': case '?': case ']':	/* globbing chars */
-      case ' ': case '\t': case '\n':		/* IFS white space */
-      case '"': case '\\':		/* quoting chars */
-      case '|': case '&': case ';':		/* shell metacharacters */
-      case '<': case '>':
-      case '#':				/* comment char */
+      if (is_shell_special(c))
 	 *r++ = '\\';
-	 *r++ = c;
-	 break;
-      default:
-	 *r++ = c;
-	 break;
-      }
+      *r++ = c;
    }
    result.set_length(r-result);
    return (result);
@@ -1026,3 +1032,18 @@ bool xtld_name_ok(const char *name)
 #endif//LIBIDN
    return false;
 }
+
+bool is_ipv4_address(const char *s)
+{
+   struct in_addr addr;
+   return inet_pton(AF_INET,s,&addr)>0;
+}
+bool is_ipv6_address(const char *s)
+{
+#if INET6
+   struct in6_addr addr;
+   return inet_pton(AF_INET6,s,&addr)>0;
+#else
+   return false;
+#endif
+}
