@@ -28,7 +28,7 @@
 #endif
 #include <stdarg.h>
 
-class xstring;
+#include "xstring.h"
 
 // expands tilde; returns pointer to static data
 const char *expand_home_relative(const char *);
@@ -123,7 +123,10 @@ static inline char *memrchr(char *buf,char c,size_t len) {
    return const_cast<char*>(memrchr(const_cast<const char*>(buf),c,len));
 }
 
-const xstring& shell_encode(const char *);
+bool is_shell_special(char c);
+const xstring& shell_encode(const char *s,int len);
+static inline const xstring& shell_encode(const char *s) { return shell_encode(s,strlen(s)); }
+static inline const xstring& shell_encode(const xstring& s) { return shell_encode(s.get(),s.length()); }
 void remove_tags(char *buf);
 void rtrim(char *s);
 
@@ -132,29 +135,10 @@ double random01();
 
 const char *get_nodename();
 
-#define ListAdd(type,chain,this,next)  \
-do {				       \
-   this->next=chain;		       \
-   chain=this;			       \
-} while(0)
-
-#define ListScan(type,chain,next) \
-   for(type *scan=chain; scan; scan=scan->next)
-
-#define ListScanPtr(type,chain,next) \
-   for(type **scan=&chain; *scan; scan=&scan[0]->next)
-
-#define ListDel(type,chain,this,next)			 \
-do {							 \
-   ListScanPtr(type,chain,next)				 \
-      if(*scan==this)					 \
-      {							 \
-	 *scan=scan[0]->next;				 \
-	 break;						 \
-      }							 \
-} while(0)
-
 const char *xidna_to_ascii(const char *name);
 bool xtld_name_ok(const char *name);
 
+bool is_ipv4_address(const char *);
+bool is_ipv6_address(const char *);
+
 #endif // MISC_H
