@@ -338,7 +338,9 @@ static ResType lftp_vars[] = {
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:no-empty-dirs",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:require-source",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"mirror:overwrite",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
+   {"sftp:auto-confirm",	 "no",	  ResMgr::BoolValidate,0},
    {"sftp:max-packets-in-flight","16",	  ResMgr::UNumberValidate,0},
    {"sftp:protocol-version",	 "6",	  ResMgr::UNumberValidate,0},
    {"sftp:size-read",		 "32k",	  ResMgr::UNumberValidate,0},
@@ -349,6 +351,7 @@ static ResType lftp_vars[] = {
    {"sftp:use-full-path",	 "yes",	  ResMgr::BoolValidate,0},
 
    {"file:charset",		 "",	  ResMgr::CharsetValidate,ResMgr::NoClosure},
+   {"file:use-lock",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
    {"dns:cache-enable",		 "yes",	  ResMgr::BoolValidate,0},
    {"dns:cache-expire",		 "1h",	  ResMgr::TimeIntervalValidate,0},
@@ -367,6 +370,7 @@ static ResType lftp_vars[] = {
    {"dns:strict-dnssec",	 "no",	  ResMgr::BoolValidate,0},
 #endif
 
+   {"fish:auto-confirm",	 "no",	  ResMgr::BoolValidate,0},
    {"fish:shell",		 "/bin/sh",0,0},
    {"fish:connect-program",	 "ssh -a -x",0,0},
    {"fish:charset",		 "",	  ResMgr::CharsetValidate,0},
@@ -391,6 +395,7 @@ static ResType lftp_vars[] = {
    {"ssl:check-hostname",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:verify-certificate",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:use-sni",		 "yes",	  ResMgr::BoolValidate,0},
+   {"ssl:priority",		 "",	  0,0},
 # if USE_OPENSSL
    {"ssl:ca-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
    {"ssl:crl-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
