@@ -40,15 +40,25 @@ private:
    xarray0(const xarray0&);	       // disable cloning
 
 protected:
-   void *get_ptr(int n) { return static_cast<char*>(buf)+n*element_size; }
+   void *get_ptr(int n) const { return static_cast<char*>(buf)+n*element_size; }
 
 public:
    xarray0(size_t e,int k=0) : element_size(e),keep_extra(k) { init(); }
    ~xarray0() { xfree(buf); }
 
-   // allocates s slots, with preferred granularity g
-   void get_space(size_t s,size_t g=32);
-   size_t get_element_size() { return element_size; }
+   // allocates s slots, with preferred granularity g, may shrink
+   void get_space_do(size_t s,size_t g=32);
+   void get_space(size_t s,size_t g=32) {
+      if(size<s+keep_extra || size/2>=s+keep_extra)
+	 get_space_do(s,g);
+   }
+   // only grows, not shrinks
+   void grow_space(size_t s,size_t g=32) {
+      if(size<s+keep_extra)
+	 get_space_do(s,g);
+   }
+
+   size_t get_element_size() const { return element_size; }
 
    int length() const { return len; }
    int count()  const { return len; }
@@ -57,7 +67,7 @@ public:
    void _nset(const void *s,int len);
    void _unset() { _nset(0,0); }
    void *_insert(int before);
-   void *_append();
+   void *_append() { grow_space(len+1); return get_ptr(len++); }
    void _remove(int i,int j);
    void _remove(int i) { _remove(i,i+1); }
    void _chop() { len--; }
@@ -178,13 +188,15 @@ public:
    xarray_s() : _RefArray<const char,T>() {}
 };
 
+// array of new'ed pointers
 template<typename T>
 class xarray_p : public xarray0
 {
    xarray_p& operator=(const xarray_p&); // make assignment fail
    xarray_p(const xarray_p&);	       // disable cloning
 
-   void dispose(int i) { delete(get_non_const()[i]); }
+   virtual void dispose(T *p) { delete p; }
+   void dispose(int i) { dispose(get_non_const()[i]); }
    void dispose(int i,int j) { while(i<j) dispose(i++); }
    void clear(int i) { get_non_const()[i]=0; }
    void clear(int i,int j) { while(i<j) clear(i++); }
@@ -197,7 +209,7 @@ public:
    T *&operator[](int i) { return get_non_const()[i]; }
    T *operator[](int i) const { return get()[i]; }
    size_t get_element_size() const { return sizeof(T*); }
-   void nset(T *const*s,int len) { dispose(0,len); _nset(s,len); if(buf) z(); }
+   void nset(T *const*s,int s_len) { dispose(0,len); _nset(s,s_len); if(buf) z(); }
    void set(const xarray_p<T> &a) { nset(a.get(),a.count()); }
    void set_length(size_t n) { dispose(n,len); _set_length(n); if(buf) z(); }
    void unset() { dispose(0,len); _unset(); }
@@ -209,8 +221,20 @@ public:
    void chop() { dispose(len-1); _chop(); }
    T *last() { return (*this)[len-1]; }
    T **borrow() { return static_cast<T**>(_borrow()); }
+   typedef int (*cmp_t)(const T**,const T**);
+   void qsort(cmp_t cmp) {
+      xarray0::qsort((qsort_cmp_t)cmp);
+   }
 };
 
+// array of malloc'ed pointers
+template<typename T>
+class xarray_m : public xarray_p<T>
+{
+   void dispose(T *p) { xfree(p); }
+public:
+   ~xarray_m() { xarray_p<T>::truncate(); }
+};
 
 
 template<typename T,class A,typename P> class _xqueue
