@@ -99,7 +99,7 @@ static inline size_t xstrlen(const char *s)
 /* this is a small and fast dynamic string class */
 /* mostly used as xstrdup/xfree replacement */
 
-enum { URL_DECODE_PLUS=1 };
+enum { URL_DECODE_PLUS=1, URL_ALLOW_8BIT=2 };
 
 class xstring0 // base class
 {
@@ -238,6 +238,8 @@ public:
    bool eq(const char *s) const { return eq(s,strlen(s)); }
    bool eq(const xstring&o) const { return eq(o.get(),o.length()); }
    bool ne(const xstring&o) const { return !eq(o); }
+   int cmp(const char *o_buf,size_t o_len) const;
+   int cmp(const xstring&o) const { return cmp(o.get(),o.length()); }
    bool chomp(char c='\n');
    void rtrim(char c=' ');
    char last_char() const { return len>0?buf[len-1]:0; }
@@ -255,9 +257,9 @@ public:
    const char *hexdump() const;
 
    xstring& url_decode(int flags=0);
-   xstring& append_url_encoded(const char *s,int len,const char *unsafe);
-   xstring& append_url_encoded(const char *s,const char *unsafe) { return append_url_encoded(s,strlen(s),unsafe); }
-   xstring& append_url_encoded(const xstring& s,const char *unsafe) { return append_url_encoded(s,s.length(),unsafe); }
+   xstring& append_url_encoded(const char *s,int len,const char *unsafe,unsigned flags=0);
+   xstring& append_url_encoded(const char *s,const char *unsafe,unsigned flags=0) { return append_url_encoded(s,strlen(s),unsafe,flags); }
+   xstring& append_url_encoded(const xstring& s,const char *unsafe,unsigned flags=0) { return append_url_encoded(s,s.length(),unsafe,flags); }
 
    xstring& append_quoted(const char *s,int len);
    xstring& append_quoted(const char *s) { return append_quoted(s,strlen(s)); }
